@@ -1,14 +1,17 @@
 -module(generator).
 -export([generate/1]).
 
-generate(N) when N >= 0 ->
-    Port = open_port({spawn, "python handler.py"},
-        [{packet, 1}, nouse_stdio, binary, {env, [{"PYTHONPATH", "../src"}]}]),
+generate(N) when is_integer(N) andalso N >= 0 ->
+    Port = open_port({spawn, "python -u handler.py"},
+        [{packet, 4}, binary, {env, [{"PYTHONPATH", "../src"}]}]),
     generate(Port, N),
     receive
         {Port, {data, Data}} ->
             port_close(Port),
             binary_to_term(Data)
+    after
+        500 ->
+            {error, timeout}
     end.
 
 generate(Port, 0) ->
