@@ -1,9 +1,9 @@
 -module(handler).
 -export([handle/1]).
 
-handle(N) when N >= 0 ->
-    Port = open_port({spawn, "python generator.py"},
-        [{packet, 1}, nouse_stdio, binary, {env, [{"PYTHONPATH", "../src"}]}]),
+handle(N) when is_integer(N) andalso N >= 0 ->
+    Port = open_port({spawn, "python -u generator.py"},
+        [{packet, 1}, binary, {env, [{"PYTHONPATH", "../src"}]}]),
     port_command(Port, term_to_binary(N)),
     handle(Port, []).
 
@@ -17,4 +17,7 @@ handle(Port, Acc) ->
                 Term ->
                     handle(Port, [Term | Acc])
             end
+    after
+        5000 ->
+            {error, timeout}
     end.
