@@ -45,20 +45,37 @@ class Atom(str):
     def __new__(cls, s):
         if len(s) > 255:
             raise ValueError("invalid atom length")
-        return str.__new__(cls, s)
+        return super(Atom, cls).__new__(cls, s)
 
     def __repr__(self):
         return "atom(%s)" % self
 
 
 class String(unicode):
-    """Erlang string or possible list."""
+    """Erlang list/string wrapper."""
 
-    def getList(self):
-        return [ord(i) for i in self]
+    def __new__(cls, s):
+        if isinstance(s, list):
+            # Raise TypeError
+            s = u"".join(unichr(i) for i in s)
+        elif not isinstance(s, unicode):
+            raise TypeError("list or unicode object expected")
+        return super(String, cls).__new__(cls, s)
 
     def __repr__(self):
-        return "string(%s)" % unicode.__repr__(self)
+        return "string(%s)" % super(String, self).__repr__()
+
+
+class BitBinary(str):
+    """Erlang bitstring whose length in bits is not a multiple of 8."""
+
+    def __new__(cls, s, bits):
+        obj = super(BitBinary, cls).__new__(cls, s)
+        obj.bits = bits
+        return obj
+
+    def __repr__(self):
+        return "bits(%s, %s)" % (self.bits, super(BitBinary, self).__repr__())
 
 
 def decode(string):
@@ -83,10 +100,10 @@ def decode_term(string):
     elif tag == 98:
         if len(tail) < 4:
             raise IncompleteData("incomplete data: %r" % string)
-        i, = unpack(">I", tail[:4])
+        i, = unpack(">i", tail[:4])
         return i, tail[4:]
     elif tag == 106:
-        return String(""), tail
+        return [], tail
     elif tag == 107:
         if len(tail) < 2:
             raise IncompleteData("incomplete data: %r" % string)
@@ -94,23 +111,18 @@ def decode_term(string):
         tail = tail[2:]
         if len(tail) < length:
             raise IncompleteData("incomplete data: %r" % string)
-        return String(tail[:length]), tail[length:]
+        return [ord(i) for i in tail[:length]], tail[length:]
     elif tag == 108:
         if len(tail) < 4:
             raise IncompleteData("incomplete data: %r" % string)
         length, = unpack(">I", tail[:4])
         tail = tail[4:]
         lst = []
-        is_string = True
         while length > 0:
             term, tail = decode_term(tail)
-            if is_string and not isinstance(term, int):
-                is_string = False
             lst.append(term)
             length -= 1
-        term, tail = decode_term(tail)
-        if is_string:
-            return String(u"".join(unichr(i) for i in lst)), tail
+        ignored, tail = decode_term(tail)
         return lst, tail
     elif tag == 109:
         if len(tail) < 4:
@@ -150,6 +162,31 @@ def decode_term(string):
         return term, tail[8:]
     elif tag == 99:
         return float(tail[:31].split("\x00", 1)[0]), tail[31:]
+    elif tag == 110 or tag == 111:
+        if tag == 110:
+            if len(tail) < 2:
+                raise IncompleteData("incomplete data: %r" % string)
+            length, sign = unpack(">BB", tail[:2])
+            tail = tail[2:]
+        else:
+            if len(tail) < 5:
+                raise IncompleteData("incomplete data: %r" % string)
+            length, sign = unpack(">IB", tail[:5])
+            tail = tail[5:]
+        if len(tail) < length:
+            raise IncompleteData("incomplete data: %r" % string)
+        n = sum(ord(c) << (8 * i) for i, c in enumerate(tail[:length]))
+        if sign:
+            n = -n
+        return n, tail[length:]
+    elif tag == 77:
+        if len(tail) < 5:
+            raise IncompleteData("incomplete data: %r" % string)
+        length, bits = unpack(">IB", tail[:5])
+        tail = tail[5:]
+        if len(tail) < length:
+            raise IncompleteData("incomplete daata: %r" % string)
+        return BitBinary(tail[:length], bits), tail[length:]
 
     raise ValueError("unsupported data tag: %i" % tag)
 
@@ -162,7 +199,7 @@ def encode_term(term):
     if isinstance(term, tuple):
         arity = len(term)
         if arity <= 255:
-            header = pack("BB", 104, arity)
+            header = 'h%c' % arity
         elif arity <= 4294967295:
             header = pack(">BI", 105, arity)
         else:
@@ -172,7 +209,15 @@ def encode_term(term):
         if not term:
             return "j"
         length = len(term)
-        if length > 4294967295:
+        if length <= 65535:
+            try:
+                bytes = "".join(chr(i) for i in term if isinstance(i, int))
+            except ValueError:
+                pass
+            else:
+                if len(bytes) == length:
+                    return pack(">BH", 107, length) + bytes
+        elif length > 4294967295:
             raise ValueError("invalid list length")
         header = pack(">BI", 108, length)
         return header + "".join(encode_term(t) for t in term) + "j"
@@ -181,25 +226,46 @@ def encode_term(term):
             return "j"
         length = len(term)
         if length <= 65535:
-            for i in term:
-                if ord(i) > 255:
-                    break
+            try:
+                bytes = term.encode("latin1")
+            except UnicodeEncodeError:
+                pass
             else:
-                return pack(">BH", 107, length) + str(term)
+                return pack(">BH", 107, length) + bytes
         return encode_term([ord(i) for i in term])
     elif isinstance(term, Atom):
         return pack(">BH", 100, len(term)) + term
+    elif isinstance(term, BitBinary):
+        # Must be before str type
+        return pack(">BIB", 77, len(term), term.bits) + term
     elif isinstance(term, str):
         length = len(term)
         if length > 4294967295:
             raise ValueError("invalid binary length")
         return pack(">BI", 109, length) + term
     elif isinstance(term, (int, long)):
-        if term > 4294967295:
-            raise ValueError("invalid integer value")
-        elif term <= 255:
-            return pack(">BB", 97, term)
-        return pack(">BI", 98, term)
+        if 0 <= term <= 255:
+            return 'a%c' % term
+        elif -2147483648 <= term <= 2147483647:
+            return pack(">Bi", 98, term)
+
+        if term >= 0:
+            sign = 0
+        else:
+            sign = 1
+            term = -term
+
+        bytes = []
+        while term > 0:
+            term, i = divmod(term, 256)
+            bytes.append(chr(i))
+
+        length = len(bytes)
+        if length <= 255:
+            return pack(">BBB", 110, length, sign) + "".join(bytes)
+        elif length <= 4294967295:
+            return pack(">BIB", 111, length, sign) + "".join(bytes)
+        raise ValueError("invalid integer value")
     elif isinstance(term, float):
         return pack(">Bd", 70, term)
 
